-- 25 Slices
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

library unisim;
use unisim.vcomponents.all;

entity CHECK is
  generic(INIT_VAL:UNSIGNED(143 downto 0);    -- Initial State
          LAST_VAL:UNSIGNED(143 downto 0));   -- Final State
  port(CLK:in STD_LOGIC;                      -- System Clock
       RESET:in STD_LOGIC:='0';               -- System Reset (not used)
       CE:in STD_LOGIC;                       -- Clock Enable
       I:in STD_LOGIC_VECTOR(8 downto 0);     -- Parallel Input Data
       O:out STD_LOGIC_VECTOR(2 downto 0));  -- Enable Outputs
end CHECK;

architecture TRICKY of CHECK is
  attribute syn_romstyle:STRING;
  
  signal CNT:UNSIGNED(10 downto 0):=(others=>'0');
  signal TC:STD_LOGIC_VECTOR(O'range):=(others=>'0');
  signal cTC:STD_LOGIC_VECTOR(TC'range);
  signal D,X:UNSIGNED(I'range);
  signal Q,Q15:UNSIGNED(I'range):=(others=>'0');
  signal ERROR:STD_LOGIC_VECTOR(O'range):=(others=>'0');
  signal cERROR:STD_LOGIC_VECTOR(ERROR'range);
  signal S3:STD_LOGIC_VECTOR(0 to (CNT'length-1)/4);
  signal C3:STD_LOGIC_VECTOR(0 to S3'length);
  signal S5:STD_LOGIC_VECTOR(0 to (I'length-1)/2);
  signal C5:STD_LOGIC_VECTOR(0 to S5'length);

  attribute syn_romstyle of X:signal is "select_rom";
begin
--  D<=UNSIGNED(I) xor (Q15 and "000000001") xor '0'&Q15(Q15'high downto Q15'low+1) xor Q15(Q15'high-1 downto Q15'low)&'0' xor Q;
  D<=UNSIGNED(I) xor '0'&Q15(Q15'high downto Q15'low+1) xor Q15(Q15'high-1 downto Q15'low)&Q15(Q15'low) xor Q;
  lk:for K in I'range generate
       function INIT_STRING(I:UNSIGNED(15 downto 0)) return STRING is
         constant HEX:STRING(1 to 16):="0123456789ABCDEF";
         variable S:STRING(1 to 4);
       begin
         for K in S'range loop
           S(K):=HEX(TO_INTEGER(I(4*(4-K)+3 downto 4*(4-K)))+1);
         end loop;
         return S;
       end;

       signal A:UNSIGNED(3 downto 0);
       signal QA,QB:STD_LOGIC;

       attribute xc_props:STRING;
       attribute xc_props of sr:label is "INIT="&INIT_STRING(INIT_VAL(16*K+15 downto 16*K));
     begin
       A<=ROTATE_LEFT(Q15,K)(A'range);
       sr:SRLC16E generic map(INIT=>TO_BITVECTOR(STD_LOGIC_VECTOR(INIT_VAL(16*K+15 downto 16*K))))
                  port map(CLK=>CLK,
                           CE=>CE,
                           D=>D(K),
                           A0=>A(0),
                           A1=>A(1),
                           A2=>A(2),
                           A3=>A(3),
                           Q=>QA,
                           Q15=>QB);
       process(CLK)
       begin
         if rising_edge(CLK) then
           if CE='1' then
             Q(K)<=QA;
             Q15(K)<=QB;
           end if;
         end if;
       end process;
     end generate;

  S3(0)<='1' when CNT(10 downto 8)="111" else '0';
  S3(1)<='1' when CNT(7 downto 4)=X"F" else '0';
  S3(2)<='1' when CNT(3 downto 0)=X"F" else '0';

  C3(C3'low)<=CE;
  ls:for K in S3'range generate
     begin
       mx:MUXCY port map(DI=>'0',CI=>C3(K),S=>S3(K),O=>C3(K+1));
     end generate;
     
--  X<=SHIFT_RIGHT(LAST_VAL,9*TO_INTEGER(CNT(3 downto 0)))(Q15'range);
  process(CNT(3 downto 0))
  begin
    case CNT(3 downto 0) is
      when "0000"=>X<=SHIFT_RIGHT(LAST_VAL,9*0)(Q15'range);
      when "0001"=>X<=SHIFT_RIGHT(LAST_VAL,9*1)(Q15'range);
      when "0010"=>X<=SHIFT_RIGHT(LAST_VAL,9*2)(Q15'range);
      when "0011"=>X<=SHIFT_RIGHT(LAST_VAL,9*3)(Q15'range);
      when "0100"=>X<=SHIFT_RIGHT(LAST_VAL,9*4)(Q15'range);
      when "0101"=>X<=SHIFT_RIGHT(LAST_VAL,9*5)(Q15'range);
      when "0110"=>X<=SHIFT_RIGHT(LAST_VAL,9*6)(Q15'range);
      when "0111"=>X<=SHIFT_RIGHT(LAST_VAL,9*7)(Q15'range);
      when "1000"=>X<=SHIFT_RIGHT(LAST_VAL,9*8)(Q15'range);
      when "1001"=>X<=SHIFT_RIGHT(LAST_VAL,9*9)(Q15'range);
      when "1010"=>X<=SHIFT_RIGHT(LAST_VAL,9*10)(Q15'range);
      when "1011"=>X<=SHIFT_RIGHT(LAST_VAL,9*11)(Q15'range);
      when "1100"=>X<=SHIFT_RIGHT(LAST_VAL,9*12)(Q15'range);
      when "1101"=>X<=SHIFT_RIGHT(LAST_VAL,9*13)(Q15'range);
      when "1110"=>X<=SHIFT_RIGHT(LAST_VAL,9*14)(Q15'range);
      when "1111"=>X<=SHIFT_RIGHT(LAST_VAL,9*15)(Q15'range);
      when others=>null;
    end case;
  end process;
  
--  S5(0)<='1' when X(1 downto 0)=D(1 downto 0) else '0';
--  S5(1)<='1' when X(3 downto 2)=D(3 downto 2) else '0';
--  S5(2)<='1' when X(5 downto 4)=D(5 downto 4) else '0';
--  S5(3)<='1' when X(7 downto 6)=D(7 downto 6) else '0';
--  S5(4)<='1' when X(8)=D(8) else '0';
  l0:LUT4 generic map(INIT=>X"8421")
          port map(I0=>D(0),I1=>D(1),I2=>X(0),I3=>X(1),O=>S5(0));
  l1:LUT4 generic map(INIT=>X"8421")
          port map(I0=>D(2),I1=>D(3),I2=>X(2),I3=>X(3),O=>S5(1));
  l2:LUT4 generic map(INIT=>X"8421")
          port map(I0=>D(4),I1=>D(5),I2=>X(4),I3=>X(5),O=>S5(2));
  l3:LUT4 generic map(INIT=>X"8421")
          port map(I0=>D(6),I1=>D(7),I2=>X(6),I3=>X(7),O=>S5(3));
  l4:LUT2 generic map(INIT=>X"9")
          port map(I0=>D(8),I1=>X(8),O=>S5(4));
  C5(C5'low)<='0';
  ll:for L in S5'range generate
     begin
       mx:MUXCY port map(DI=>C3(C3'high-1),CI=>C5(L),S=>S5(L),O=>C5(L+1));
     end generate;

  cTC<=(others=>'1') when C3(C3'high)='1' else TC;
  cERROR<=(others=>'1') when C5(C5'high)='1' else ERROR;
  process(CLK,RESET)
  begin
    if RESET='1' then
      CNT<=(others=>'0');
      TC<=(others=>'0');
      ERROR<=(others=>'0');
      O<=(others=>'0');
    elsif rising_edge(CLK) then
      TC<=cTC;
      if CE='1' then
        if TC="000" then
          CNT<=CNT+1;
        end if;
        ERROR<=cERROR;
        O<=TC and not ERROR;
      end if;
    end if;
  end process;
  --O=iO;
end TRICKY;