--------------------------------------------------------------------------------
-- Copyright (c) 1996-2005 Xilinx, Inc.
-- All Rights Reserved
--------------------------------------------------------------------------------
--   ____  ____
--  /   /\/   /
-- /___/  \  /   Vendor: Xilinx
-- \   \   \/    Version: 3.0
--  \   \        Filename: ifftest.vhd
--  /   /        Date Last Modified:  Thursday April 21 2005
-- /___/   /\    Date Created: June 6 2004
-- \   \  /  \
--  \___\/\___\
-- 
--Device: Xilinx 
--Library: UNISIM
--Purpose: 
--  FPGA Bitstream Copy Protection Reference Design Using the
--  Maxim DS2432 Secure Serial EEPROM - Check DS2432 Version
--Reference:
--   www.xilinx.com/bvdocs/appnotes/xapp780.pdf 
--Revision History:
--    Rev 2.0.0 - Initial release, catalinb, June 6 2004.
--    Rev 3.0.0 - Added ring oscillator, PicoBlaze code tamper detection,
--						catalinb, March 8 2005
--		Rev 3.0.1 - XST support; Added multiple tamper detection signals. 
--						Added multiple FOE signals, shalins, April 21 2005
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
--              provided to you "as is". Xilinx and its licensors make and you 
--              receive no warranties or conditions, express, implied, 
--              statutory or otherwise, and Xilinx specifically disclaims any 
--              implied warranties of merchantability, non-infringement, or 
--              fitness for a particular purpose. Xilinx does not warrant that 
--              the functions contained in these designs will meet your 
--              requirements, or that the operation of these designs will be 
--              uninterrupted or error free, or that defects in the Designs 
--              will be corrected. Furthermore, Xilinx does not warrant or 
--              make any representations regarding use or the results of the 
--              use of the designs in terms of correctness, accuracy, 
--              reliability, or otherwise. 
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its 
--              licensors be liable for any loss of data, lost profits, cost 
--              or procurement of substitute goods or services, or for any 
--              special, incidental, consequential, or indirect damages 
--              arising from the use or operation of the designs or 
--              accompanying documentation, however caused and on any theory 
--              of liability. This limitation will apply even if Xilinx 
--              has been advised of the possibility of such damage. This 
--              limitation shall apply not-withstanding the failure of the 
--              essential purpose of any limited remedies herein. 
--
--  Copyright  2005 Xilinx, Inc.
--  All rights reserved 
-- 
--------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.NUMERIC_STD.all;

library unisim;
use unisim.vcomponents.all;

entity IFFTEST is
  port(clkin:in STD_LOGIC; 	-- input clock only used for PicoBlaze interrupt
       RESET:in STD_LOGIC;		-- Active HIGH reset for system
       INIT:out STD_LOGIC;		--	Indicates PicoBlaze code tampering
       FOE0:out STD_LOGIC;	   --	FOE signal 0, active high
		 FOE1:out STD_LOGIC;		--	FOE signal 1, active high
		 FOE2:out STD_LOGIC;		--	FOE signal 2, active high
		 FOE3:out STD_LOGIC;		--	FOE signal 3, active high
		 FOE4:out STD_LOGIC;		--	FOE signal 4, active high
		 FOE5:out STD_LOGIC;		--	FOE signal 5, active high
		 FOE6:out STD_LOGIC;		--	FOE signal 6, active high
       YELLOW_LED:out STD_LOGIC; --	yellown LED, active high
       RED_LED:out STD_LOGIC;    --	red LED, active high
       IB:inout STD_LOGIC);	--signal connected to DS2432 device
end IFFTEST;

architecture PICOBLAZE of IFFTEST is
  component kcpsm2 
    generic (Spartan3:BOOLEAN:=FALSE);
    port(address:out STD_LOGIC_VECTOR(9 downto 0);
         instruction:in STD_LOGIC_VECTOR(17 downto 0);
         port_id:out STD_LOGIC_VECTOR(7 downto 0);
         write_strobe:out STD_LOGIC;
         out_port:out STD_LOGIC_VECTOR(7 downto 0);
         read_strobe:out STD_LOGIC;
         in_port:in STD_LOGIC_VECTOR(7 downto 0);
         interrupt:in STD_LOGIC;
         reset:in STD_LOGIC;
         clk:in STD_LOGIC);
  end component;

  component IFF1         -- use for V2/V2Pro/S3
    port(address:in STD_LOGIC_VECTOR(9 downto 0);
         instruction:out STD_LOGIC_VECTOR(17 downto 0);
         di:in STD_LOGIC_VECTOR(8 downto 0):="111111111";
         do:out STD_LOGIC_VECTOR(8 downto 0);
         wa:in STD_LOGIC_VECTOR(10 downto 0):="11111111111";
         we:in STD_LOGIC:='0';
         nclk:in STD_LOGIC;
         clk:in STD_LOGIC);
  end component;

  component RND
    generic(Spartan3:BOOLEAN:=FALSE);
    port(CLK:in STD_LOGIC;     -- System Clock
         O:out STD_LOGIC);     -- Random Serial Output Data
  end component;

  component CHECK
    generic(INIT_VAL:UNSIGNED(143 downto 0);    -- Initial State
            LAST_VAL:UNSIGNED(143 downto 0));   -- Final State
    port(CLK:in STD_LOGIC;                      -- System Clock
         RESET:in STD_LOGIC:='0';               -- System Reset (not used)
         CE:in STD_LOGIC;                       -- Clock Enable
         I:in STD_LOGIC_VECTOR(8 downto 0);     -- Parallel Input Data
         O:out STD_LOGIC_VECTOR(2 downto 0));  -- Enable Outputs
  end component;

  component RINGOSC
    generic(FPGA:INTEGER:=3;         -- 1 for Virtex2Pro, 2 for Virtex2, 3 for Spartan3, 4 for Virtex4
-- 3S-4
            tILO:TIME:=608 ps;
            tNETX:TIME:=1601 ps;
            tNETY:TIME:=1551 ps);
-- 2VP-5
--            tILO:TIME:=335 ps;
--            tNETX:TIME:=1115 ps;
--            tNETY:TIME:=1083 ps);
    port(CLK:out STD_LOGIC);
  end component;

  type TKEY is array(0 to 7) of STD_LOGIC_VECTOR(7 downto 0);
-- This is the secret master key - replace it with your own!
  constant KEY:TKEY:=(X"01",X"23",X"45",X"67",X"89",X"AB",X"CD",X"EF");

  type TSCRKEY is array(0 to 3) of STD_LOGIC_VECTOR(7 downto 0);
-- This is the PicoBlaze code scrambling key - replace it with your own!
  constant SCRKEY:TSCRKEY:=(X"78",X"56",X"34",X"12");

-- Code Signature Keys
  --constant INIT_VAL:UNSIGNED(143 downto 0):=X"5AC9BCE7416F86CA06265E3FD6CF7A75D37D";   -- Initial State
  --constant LAST_VAL:UNSIGNED(143 downto 0):=X"2DC740502A6449F81401CA8C1E129772AA21";   -- Final State
  --shalin added	  4
  --constant INIT_VAL:UNSIGNED(143 downto 0):=X"459DC095C6D3A1D321484E9532027A716A64";   -- Initial State
  --constant LAST_VAL:UNSIGNED(143 downto 0):=X"57852ACF55EE74A3D88F4BDE077EC4B8016C";   -- Final State

    constant INIT_VAL:UNSIGNED(143 downto 0):=X"99A399A399A399A399A399A399A399A399A3";   -- Initial State
  constant LAST_VAL:UNSIGNED(143 downto 0):=X"F2FBDBB80282D1CDA338701B0E60FE0843A5";   -- Final State

  signal address:STD_LOGIC_VECTOR(9 downto 0);
  signal instruction:STD_LOGIC_VECTOR(17 downto 0);
  signal port_id:STD_LOGIC_VECTOR(7 downto 0);
  signal out_port:STD_LOGIC_VECTOR(7 downto 0);
  signal in_port:STD_LOGIC_VECTOR(7 downto 0);
  signal write_strobe:STD_LOGIC;
  signal read_strobe:STD_LOGIC;
  signal interrupt:STD_LOGIC:='0';
  signal INTCNT:UNSIGNED(6 downto 0):=(others=>'0');

-- MAXCNT must be equal to your system clock frequency expressed in MHz
  --constant MAXCNT:INTEGER:=25; -- for a 25MHz CLK
  constant MAXCNT:INTEGER:=50; -- for a 25MHz CLK

  signal di:STD_LOGIC_VECTOR(8 downto 0);
  signal do:STD_LOGIC_VECTOR(8 downto 0);
  signal wa:STD_LOGIC_VECTOR(10 downto 0);
  signal we:STD_LOGIC;

  signal RESET_int:STD_LOGIC;
  signal ibo:STD_LOGIC;
  signal CLK:STD_LOGIC;
  signal ACLK_f,ACLK:STD_LOGIC;
  signal RBIT:STD_LOGIC;
  signal RBYTE:STD_LOGIC_VECTOR(7 downto 0);
  signal di8,do8:STD_LOGIC_VECTOR(1 downto 0):="00";
  signal page:STD_LOGIC_VECTOR(3 downto 0):=X"0";
  signal CE:STD_LOGIC;
  signal ENABLE:STD_LOGIC_VECTOR(2 downto 0);
-- user interface signals
  signal IFF:STD_LOGIC; -- rising edge requests PicoBlaze to authenticate DS2432
  signal nFOE:STD_LOGIC:='0';
  signal FOE, FOE0_int, FOE1_int, FOE2_int, FOE3_int, FOE4_int, FOE5_int, FOE6_int:STD_LOGIC:='1'; -- PicoBlaze responds with a '0' if DS2432 holds a valid key and a '1' if not
  --signal FOE2:STD_LOGIC_VECTOR(6 downto 0);
  -------------------------------------------------------------------
  --
  --  ICON core component declaration
  --
  -------------------------------------------------------------------
  component icon
    port(control0    :   out std_logic_vector(35 downto 0);
         control1    :   out std_logic_vector(35 downto 0));
  end component;

  -------------------------------------------------------------------
  --
  --  ILA core component declaration
  --
  -------------------------------------------------------------------
  component ila
    port(control     : in    std_logic_vector(35 downto 0);
         clk         : in    std_logic;
         trig0       : in    std_logic_vector(37 downto 0));
  end component;

  -------------------------------------------------------------------
  --
  --  VIO core component declaration
  --
  -------------------------------------------------------------------
  component vio
    port(control     : in    std_logic_vector(35 downto 0);
         async_in    : in    std_logic_vector(0 downto 0);
         async_out   :   out std_logic_vector(0 downto 0));
  end component;

  -------------------------------------------------------------------
  --
  --  ILA core signal declarations
  --
  -------------------------------------------------------------------
  signal control0   : std_logic_vector(35 downto 0);

  -------------------------------------------------------------------
  --
  --  VIO core signal declarations
  --
  -------------------------------------------------------------------
  signal control1   : std_logic_vector(35 downto 0);
  signal async_in   : std_logic_vector(0 downto 0);
  signal async_out  : std_logic_vector(0 downto 0);

  -------------------------------------------------------------------
  --
  --  ICON core compiler-specific attributes
  --
  -------------------------------------------------------------------
  attribute syn_black_box : boolean;
  attribute syn_black_box of icon : component is TRUE;
  attribute syn_noprune : boolean;
  attribute syn_noprune of icon : component is TRUE;

  -------------------------------------------------------------------
  --
  --  ILA core compiler-specific attributes
  --
  -------------------------------------------------------------------
  attribute syn_black_box of ila : component is TRUE;
  attribute syn_noprune of ila : component is TRUE;
  
  -------------------------------------------------------------------
  --
  --  VIO core compiler-specific attributes
  --
  -------------------------------------------------------------------
  attribute syn_black_box of vio : component is TRUE;
  attribute syn_noprune of vio : component is TRUE;
begin
  CLK<=CLKIN;
  --RESET<=not RESETn; -- we use the RESETn button to request an IFF check
  --Shalin Change 4/19/05
  RESET_int<=RESET; -- we use the RESETn button to request an IFF check
  INIT<=not ENABLE(0);  -- the INIT LED indicates and attempt to change the PicoBlaze code
  --GREEN_LED<=not FOE and not IFF;  -- this is the pass condition
  FOE0<=not FOE0_int and not IFF;  -- this is the pass condition
  FOE1<=not FOE1_int and not IFF;  -- this is the pass condition
  FOE2<=not FOE2_int and not IFF;  -- this is the pass condition
  FOE3<=not FOE3_int and not IFF;  -- this is the pass condition
  FOE4<=not FOE4_int and not IFF;  -- this is the pass condition
  FOE5<=not FOE5_int and not IFF;  -- this is the pass condition
  FOE6<=not FOE6_int and not IFF;  -- this is the pass condition
 

  YELLOW_LED<=IFF;                 -- this is the IFF request
  RED_LED<=FOE and not IFF;        -- this is the fail condition
-- generate  a 1us interrupt for PicoBlaze, used to time the 1-wire interface for the DS2432
  process(CLK)
  begin
    if rising_edge(CLK) then
      if INTCNT=2 then
        interrupt<='1';
      elsif INTCNT=0 then
        interrupt<='0';
      end if;
      if INTCNT=0 then
        INTCNT<=TO_UNSIGNED(MAXCNT-1,INTCNT'length);
      else
        INTCNT<=INTCNT-1;
      end if;
    end if;
  end process;
-- Asynchronous Clock for PicoBlaze
  ro:RINGOSC generic map(FPGA=>3)
             port map(CLK=>ACLK_f);
  bg:BUFG port map(I=>ACLK_f,O=>ACLK);
-- serial true random number generator
--  rb:RND generic map(Spartan3=>FALSE)  -- use for V2/V2Pro
  rb:RND  generic map(Spartan3=>TRUE)   -- use for S3
         port map(CLK=>ACLK,
                  O=>RBIT);
-- create a random byte from the random serial bit stream
  process(ACLK)
  begin
    if rising_edge(ACLK) then
      RBYTE<=RBYTE(RBYTE'high-1 downto RBYTE'low)&RBIT;
    end if;
  end process;
-- the Spartan3 generic must be TRUE for VE/S2E/S3 and FALSE for V2/V2Pro
  --pb:kcpsm2 generic map(Spartan3=>FALSE)  -- use for V2/V2Pro
  pb:kcpsm2 generic map(Spartan3=>TRUE)   -- use for VE/S2E/S3
            port map(address=>address,
                     instruction=>instruction,
                     port_id=>port_id,
                     write_strobe=>write_strobe,
                     out_port=>out_port,
                     read_strobe=>read_strobe,
                     in_port=>in_port,
                     interrupt=>interrupt,
                     reset=>'0',  --RESET,
                     clk=>ACLK);
-- for a VE/S2E target you must use the vir2spart.pl Perl script which will generate
-- a PICOCODE.VHD file - use it instead of the SHA1.VHD produced by KCPSM2.EXE
  br:IFF1 port map(address=>address,     -- use for V2/V2Pro/S3
--  br:PICOCODE port map(address=>address,  -- use for VE/S2E
                   instruction=>instruction,
                   di=>di,
                   do=>do,
                   wa=>wa,
                   we=>we,
                   nclk=>ACLK,
                   clk=>ACLK);

  CE<=read_strobe when ENABLE(0)='0' else '0';
  ck:CHECK generic map(INIT_VAL=>INIT_VAL,
                       LAST_VAL=>LAST_VAL)
           port map(CLK=>ACLK,
                    CE=>CE,
                    I=>do,
                    O=>ENABLE);

  process(ACLK)
  begin
    if rising_edge(ACLK) then
      if read_strobe='1' and port_id(7)='1' then
        if port_id(0)='0' then
          do8(0)<=do(8);
        else
          do8(1)<=do(8);
        end if;
      end if;
      if write_strobe='1' and port_id(7 downto 0)=X"04" then
        di8<=out_port(1 downto 0);
      end if;
      if write_strobe='1' and port_id(7 downto 0)=X"05" then
        page<=out_port(3 downto 0);
      end if;
    end if;
  end process;

  in_port<=do(7 downto 0) when port_id(7)='1' else
           "000000"&IFF&ib when port_id(7)='0' and port_id(4 downto 0)="00010" else
           RBYTE when port_id(7)='0' and port_id(4 downto 0)="00011" else
           "000000"&do8 when port_id(4 downto 0)="00100" else
           "0000"&page when port_id(4 downto 0)="00101" else
           KEY(TO_INTEGER(UNSIGNED(port_id(2 downto 0)))) when port_id(4 downto 3)="01" and ENABLE(1)='1' else
           SCRKEY(TO_INTEGER(UNSIGNED(port_id(1 downto 0)))) when port_id(4 downto 3)="10" and ENABLE(2)='1' else X"00";
  process(ACLK)
  begin
    if rising_edge(ACLK) then
-- we are using RESET for something else now
--      if RESET='1' then
--        ibo<='1';
--        FOE<='1';
--      elsif write_strobe='1' and port_id=X"02" then
      if write_strobe='1' and port_id=X"02" and ENABLE(1)='1' then
        ibo<=out_port(0);
        nFOE<=not out_port(1);
		  FOE0_int<= out_port(1);
		  FOE1_int<= out_port(2);
		  FOE2_int<= out_port(3);
		  FOE3_int<= out_port(4);
		  FOE4_int<= out_port(5);
		  FOE5_int<= out_port(6);
		  FOE6_int<= out_port(6);
      end if;
    end if;
  end process;
  FOE<=not nFOE;
  IB<='0' when ibo='0' else 'Z';
    
  di<=di8(TO_INTEGER(UNSIGNED(port_id(0 downto 0))))&out_port;
  we<=write_strobe when port_id(7)='1' else '0';
  wa<=page(3 downto 0)&port_id(6 downto 0);

  -------------------------------------------------------------------
  --
  --  ICON core instance
  --
  -------------------------------------------------------------------
  i_icon:icon port map(control0 => control0,
                       control1 => control1);
  -------------------------------------------------------------------
  --
  --  ILA core instance
  --
  -------------------------------------------------------------------
  i_ila:ila port map(control             => control0,
                     clk                 => ACLK,
                     trig0(9 downto 0)   => address,
                     trig0(17 downto 10) => port_id,
                     trig0(25 downto 18) => in_port,
                     trig0(33 downto 26) => out_port,
                     trig0(34)           => read_strobe,
                     trig0(35)           => write_strobe,
                     trig0(36)           => IFF,
                     trig0(37)           => FOE);

  -------------------------------------------------------------------
  --
  --  VIO core instance
  --
  -------------------------------------------------------------------
  async_in(0)<=FOE;
  i_vio:vio port map(control   => control1,
                     async_in  => async_in,
                     async_out => async_out);
--  IFF<=async_out(0);
-- request a program by pushing the button or trough ChipScope
  IFF<=RESET_int or async_out(0);
end PICOBLAZE;