-- The CLK frequency is guaranteed to be between 28.6MHz and 95.4MHz in any Spartan3 FPGA
-- The CLK frequency is guaranteed to be between 21.8MHz and 72.7MHz in any V2Pro FPGA
library ieee;
use ieee.std_logic_1164.all;

library unisim;
use unisim.vcomponents.all;

entity RINGOSC is
  generic(FPGA:INTEGER:=3;         -- 1 for Virtex2Pro, 2 for Virtex2, 3 for Spartan3, 4 for Virtex4
-- 3S-4
          tILO:TIME:=608 ps;
          tNETX:TIME:=1601 ps;
          tNETY:TIME:=1551 ps);
-- 2VP-5
--          tILO:TIME:=335 ps;
--          tNETX:TIME:=1115 ps;
--          tNETY:TIME:=1083 ps);
  port(CLK:out STD_LOGIC);
end RINGOSC;

architecture ASYNC of RINGOSC is
  attribute rloc:STRING;
  attribute bel:STRING;
  attribute route:STRING;
  attribute s:STRING;
  attribute syn_keep:BOOLEAN;
begin
  i1:if FPGA=1 generate -- Virtex2Pro
       signal X,Y,DX,DY:STD_LOGIC:='0';
       signal Q0,Q1,Q2,Q3:STD_LOGIC:='0';
       signal nQ0:STD_LOGIC;

       attribute rloc of l0:label is "X0Y2";
       attribute bel of l0:label is "F";
       attribute s of X:signal is "yes";
       attribute syn_keep of X:signal is TRUE;
       attribute route of X:signal is "{2;1;-5!-1;-42032;-22600;14;3;8;30;13!0;-1848;5513;3!1;4088;-634;2!2;-499;-4623;3!3;-3077;1367;2!4;1184;-1378;0!5;152;-365;4;3;8;18!}";
       attribute rloc of l1:label is "X0Y2";
       attribute bel of l1:label is "G";
       attribute s of Y:signal is "yes";
       attribute syn_keep of Y:signal is TRUE;
       attribute route of Y:signal is "{2;1;-5!-1;-42032;-22568;14;3;8;33;13!0;-2075;1440;24!0;438;2056;"&
                                      "3!1;-284;7232;29!2;4177;-7901;2!3;-2096;-2427;10;3;10;4!3;-160;-688;10;3;"&
                                      "8!4;1813;416;3!7;-3021;-671;2!8;1088;684;0!9;120;-365;4;3;8;11!}";
       attribute rloc of f0:label is "X0Y2";
       attribute bel of f0:label is "FFX";
       attribute rloc of f1:label is "X0Y2";
       attribute bel of f1:label is "FFY";
       attribute rloc of f2:label is "X1Y2";
       attribute bel of f2:label is "FFX";
       attribute rloc of f3:label is "X1Y2";
       attribute bel of f3:label is "FFY";
     begin
       DY<=transport '1' after tILO+tNETY when Y='X' or Y='U' else Y after tILO+tNETY;
       l0:LUT1 generic map(INIT=>"01")
               port map(I0=>DY,O=>X);
       DX<=transport '1' after tILO+tNETX when X='X' or X='U' else X after tILO+tNETX;
       l1:LUT1 generic map(INIT=>"10")
               port map(I0=>DX,O=>Y);
  
       nQ0<=not Q0;
       f0:FD port map(C=>Y,D=>Q3,Q=>Q0);
       f1:FD port map(C=>Y,D=>nQ0,Q=>Q1);
       f2:FD port map(C=>Y,D=>Q1,Q=>Q2);
       f3:FD port map(C=>Y,D=>Q2,Q=>Q3);
       CLK<=Q1;
     end generate;

  i3:if FPGA=3 generate -- Spartan3
       signal X,Y,DX,DY:STD_LOGIC:='0';
       signal Q0,Q1:STD_LOGIC:='0';
       signal nQ0:STD_LOGIC;

       attribute rloc of l0:label is "X0Y2";
       attribute bel of l0:label is "F";
       attribute s of X:signal is "yes";
       attribute syn_keep of X:signal is TRUE;
       attribute route of X:signal is "{2;1;-4!-1;-32584;-40720;14;15;20;18;13!0;-1776;5909;3!1;4064;-1066;2!2;-511;-4635;3!3;-2953;1265;2!4;1009;-1593;0!5;167;0;4;15;20;10!}";
       attribute rloc of l1:label is "X0Y2";
       attribute bel of l1:label is "G";
       attribute s of Y:signal is "yes";
       attribute syn_keep of Y:signal is TRUE;
       attribute route of Y:signal is "{2;1;-4!-1;-32584;-40688;14;15;20;21;13!0;2256;-549;2!0;-3893;"&
                                      "4045;3!1;1637;-3784;10;15;20;4!2;2256;3115;2!4;-443;-2699;3!5;-3021;-671;"&
                                      "2!6;1041;319;0!7;167;0;4;15;20;7!}";
       attribute rloc of f0:label is "X0Y2";
       attribute bel of f0:label is "FFX";
       attribute rloc of f1:label is "X0Y2";
       attribute bel of f1:label is "FFY";
     begin
       DY<=transport '1' after tILO+tNETY when Y='X' or Y='U' else Y after tILO+tNETY;
       l0:LUT1 generic map(INIT=>"01")
               port map(I0=>DY,O=>X);
       DX<=transport '1' after tILO+tNETX when X='X' or X='U' else X after tILO+tNETX;
       l1:LUT1 generic map(INIT=>"10")
               port map(I0=>DX,O=>Y);
       
       nQ0<=not Q0;
       f0:FD port map(C=>Y,D=>Q1,Q=>Q0);
       f1:FD port map(C=>Y,D=>nQ0,Q=>Q1);
       CLK<=Q1;
     end generate;
end ASYNC;