--******************************************************************************
--
--  Xilinx, Inc. 2004                 www.xilinx.com
--
--  FPGA Bitstream Copy Protection Reference Design Using the
--  Maxim DS2432 Secure Serial EEPROM - Load DS2432 Key Version
--
--*******************************************************************************
--
--  File name :       LOADTEST.VHD
--
--  Description :     This is the version of the bitstream copy protection
--                    reference design that loads the secret key into DS2432
--
--  Date - revision : 04/15/2004
--
--  Author :          Catalin Baetoniu
--
--  Contact :         catalinb@xilinx.com - This design is not supported by the hotline
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
--              provided to you "as is". Xilinx and its licensors make and you 
--              receive no warranties or conditions, express, implied, 
--              statutory or otherwise, and Xilinx specifically disclaims any 
--              implied warranties of merchantability, non-infringement, or 
--              fitness for a particular purpose. Xilinx does not warrant that 
--              the functions contained in these designs will meet your 
--              requirements, or that the operation of these designs will be 
--              uninterrupted or error free, or that defects in the Designs 
--              will be corrected. Furthermore, Xilinx does not warrant or 
--              make any representations regarding use or the results of the 
--              use of the designs in terms of correctness, accuracy, 
--              reliability, or otherwise. 
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its 
--              licensors be liable for any loss of data, lost profits, cost 
--              or procurement of substitute goods or services, or for any 
--              special, incidental, consequential, or indirect damages 
--              arising from the use or operation of the designs or 
--              accompanying documentation, however caused and on any theory 
--              of liability. This limitation will apply even if Xilinx 
--              has been advised of the possibility of such damage. This 
--              limitation shall apply not-withstanding the failure of the 
--              essential purpose of any limited remedies herein. 
--
--  Copyright  2004 Xilinx, Inc.
--  All rights reserved 
-- 
--*****************************************************************************

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.NUMERIC_STD.all;

entity LOADTEST is
  port(clkin:in STD_LOGIC;
       --RESETn:in STD_LOGIC;
		 PASS: out STD_LOGIC;
		 PRGM: in STD_LOGIC;
		 RESET:in STD_LOGIC;
       IB:inout STD_LOGIC);
end LOADTEST;

architecture PICOBLAZE of LOADTEST is
  component kcpsm2 
    generic (Spartan3:BOOLEAN:=FALSE);
    port(address:out STD_LOGIC_VECTOR(9 downto 0);
         instruction:in STD_LOGIC_VECTOR(17 downto 0);
         port_id:out STD_LOGIC_VECTOR(7 downto 0);
         write_strobe:out STD_LOGIC;
         out_port:out STD_LOGIC_VECTOR(7 downto 0);
         read_strobe:out STD_LOGIC;
         in_port:in STD_LOGIC_VECTOR(7 downto 0);
         interrupt:in STD_LOGIC;
         reset:in STD_LOGIC;
         clk:in STD_LOGIC);
  end component;

  component LOAD1         -- use for V2/V2Pro/S3
--  component PICOCODE      -- use for VE/S2E
    port(address:in STD_LOGIC_VECTOR(9 downto 0);
         instruction:out STD_LOGIC_VECTOR(17 downto 0);
         di:in STD_LOGIC_VECTOR(8 downto 0):="111111111";
         do:out STD_LOGIC_VECTOR(8 downto 0);
         wa:in STD_LOGIC_VECTOR(10 downto 0):="11111111111";
         we:in STD_LOGIC:='0';
         nclk:in STD_LOGIC;
         clk:in STD_LOGIC);
  end component;

  component RND
    generic(Spartan3:BOOLEAN:=FALSE);
    port(CLK:in STD_LOGIC;     -- System Clock
         O:out STD_LOGIC);     -- Random Serial Output Data
  end component;

  type TKEY is array(0 to 7) of STD_LOGIC_VECTOR(7 downto 0);
-- This is the secret master key - replace it with your own!
  constant KEY:TKEY:=(X"01",X"23",X"45",X"67",X"89",X"AB",X"CD",X"EF");

  signal address:STD_LOGIC_VECTOR(9 downto 0);
  signal instruction:STD_LOGIC_VECTOR(17 downto 0);
  signal port_id:STD_LOGIC_VECTOR(7 downto 0);
  signal out_port:STD_LOGIC_VECTOR(7 downto 0);
  signal in_port:STD_LOGIC_VECTOR(7 downto 0);
  signal write_strobe:STD_LOGIC;
  signal read_strobe:STD_LOGIC;
  signal interrupt:STD_LOGIC:='0';
  signal INTCNT:UNSIGNED(6 downto 0):=(others=>'0');

-- MAXCNT must be equal to your system clock frequency expressed in MHz
--  constant MAXCNT:INTEGER:=24; -- for a 24MHz CLK
--  constant MAXCNT:INTEGER:=125; -- for a 125MHz CLK
  constant MAXCNT:INTEGER:=50; -- for a 50MHz CLK

  signal di:STD_LOGIC_VECTOR(8 downto 0);
  signal do:STD_LOGIC_VECTOR(8 downto 0);
  signal wa:STD_LOGIC_VECTOR(10 downto 0);
  signal we:STD_LOGIC;

  --signal RESET:STD_LOGIC;
  signal ibo:STD_LOGIC;
  signal CLK:STD_LOGIC;
  signal RBIT:STD_LOGIC;
  signal RBYTE:STD_LOGIC_VECTOR(7 downto 0);
-- user interface signals
  --signal PRGM:STD_LOGIC; -- rising edge requests PicoBlaze to program secret key into DS2432
  --signal PASS:STD_LOGIC; -- PicoBlaze responds with '1' for PASS and '0' for FAIL
  -------------------------------------------------------------------
  --
  --  ICON core component declaration
  --
  -------------------------------------------------------------------
  component icon
    port(control0    :   out std_logic_vector(35 downto 0);
         control1    :   out std_logic_vector(35 downto 0));
  end component;

  -------------------------------------------------------------------
  --
  --  ILA core component declaration
  --
  -------------------------------------------------------------------
  component ila
    port(control     : in    std_logic_vector(35 downto 0);
         clk         : in    std_logic;
         trig0       : in    std_logic_vector(37 downto 0));
  end component;

  -------------------------------------------------------------------
  --
  --  VIO core component declaration
  --
  -------------------------------------------------------------------
  component vio
    port(control     : in    std_logic_vector(35 downto 0);
         async_in    : in    std_logic_vector(0 downto 0);
         async_out   :   out std_logic_vector(0 downto 0));
  end component;

  -------------------------------------------------------------------
  --
  --  ILA core signal declarations
  --
  -------------------------------------------------------------------
  signal control0   : std_logic_vector(35 downto 0);

  -------------------------------------------------------------------
  --
  --  VIO core signal declarations
  --
  -------------------------------------------------------------------
  signal control1   : std_logic_vector(35 downto 0);
  signal async_in   : std_logic_vector(0 downto 0);
  signal async_out  : std_logic_vector(0 downto 0);
begin
  CLK<=CLKIN;
  --RESET<=not RESETn;
-- generate  a 1us interrupt for PicoBlaze, used to time the 1-wire interface for the DS2432
  process(CLK)
  begin
    if rising_edge(CLK) then
      if INTCNT=2 then
        interrupt<='1';
      elsif INTCNT=0 then
        interrupt<='0';
      end if;
      if INTCNT=0 then
        INTCNT<=TO_UNSIGNED(MAXCNT-1,INTCNT'length);
      else
        INTCNT<=INTCNT-1;
      end if;
    end if;
  end process;
-- serial true random number generator
--  rb:RND generic map(Spartan3=>FALSE)  -- use for V2/V2Pro
  rb:RND  generic map(Spartan3=>TRUE)   -- use for S3
          port map(CLK=>CLK,
                  O=>RBIT);
-- create a random byte from the random serial bit stream
  process(CLK)
  begin
    if rising_edge(CLK) then
      RBYTE<=RBYTE(RBYTE'high-1 downto RBYTE'low)&RBIT;
    end if;
  end process;
-- the Spartan3 generic must be TRUE for VE/S2E/S3 and FALSE for V2/V2Pro
--  pb:kcpsm2 generic map(Spartan3=>FALSE)  -- use for V2/V2Pro
  pb:kcpsm2 generic map(Spartan3=>TRUE)   -- use for VE/S2E/S3
            port map(address=>address,
                     instruction=>instruction,
                     port_id=>port_id,
                     write_strobe=>write_strobe,
                     out_port=>out_port,
                     read_strobe=>read_strobe,
                     in_port=>in_port,
                     interrupt=>interrupt,
                     reset=>RESET,
                     clk=>CLK);
-- for a VE/S2E target you must use the vir2spart.pl Perl script which will generate
-- a PICOCODE.VHD file - use it instead of the SHA1.VHD produced by KCPSM2.EXE
  br:LOAD1 port map(address=>address,     -- use for V2/V2Pro/S3
--  br:PICOCODE port map(address=>address,  -- use for VE/S2E
                   instruction=>instruction,
                   di=>di,
                   do=>do,
                   wa=>wa,
                   we=>we,
                   nclk=>CLK,
                   clk=>CLK);

  in_port<=do(7 downto 0) when port_id(7)='1' else
           "000000"&PRGM&ib when port_id(7)='0' and port_id(3 downto 0)="0010" else
           RBYTE when port_id(7)='0' and port_id(3 downto 0)="0011" else
           KEY(TO_INTEGER(UNSIGNED(port_id(2 downto 0)))) when port_id(3)='1' else X"00";
  process(CLK)
  begin
    if rising_edge(CLK) then
      if RESET='1' then
        ibo<='1';
        PASS<='0';
      elsif write_strobe='1' and port_id=X"02" then
        ibo<=out_port(0);
        PASS<=out_port(1);
      end if;
    end if;
  end process;
  IB<='0' when ibo='0' else 'Z';
    
  di<='1'&out_port;
  we<=write_strobe when port_id(7)='1' else '0';
  wa<="111"&port_id;


end PICOBLAZE;

